'use strict';

var _extends = require('babel-runtime/helpers/extends')['default'];

var _regeneratorRuntime = require('babel-runtime/regenerator')['default'];

var _Promise = require('babel-runtime/core-js/promise')['default'];

var _interopRequireDefault = require('babel-runtime/helpers/interop-require-default')['default'];

exports.__esModule = true;

var _this = this;

var _axios = require('axios');

var _axios2 = _interopRequireDefault(_axios);

var _lodashIsplainobject = require('lodash.isplainobject');

var _lodashIsplainobject2 = _interopRequireDefault(_lodashIsplainobject);

var _CALL_API = require('./CALL_API');

var _CALL_API2 = _interopRequireDefault(_CALL_API);

var _validation = require('./validation');

var _errors = require('./errors');

var _util = require('./util');

/**
 * A Redux middleware that processes RSAA actions.
 *
 * @type {ReduxMiddleware}
 * @access public
 */
var apiMiddleware = function apiMiddleware(axiosInstance) {
  return function (_ref) {
    var getState = _ref.getState;

    var axios = axiosInstance || _axios2['default'];

    return function (next) {
      return function callee$3$0(action) {
        var validationErrors, _callAPI, _requestType, callAPI, endpoint, headers, method, body, credentials, bailout, types, list, dynamicEndpoint, _normalizeTypeDescriptors, requestType, successType, failureType, callList, res;

        return _regeneratorRuntime.async(function callee$3$0$(context$4$0) {
          while (1) switch (context$4$0.prev = context$4$0.next) {
            case 0:
              if (_validation.isRSAA(action)) {
                context$4$0.next = 2;
                break;
              }

              return context$4$0.abrupt('return', next(action));

            case 2:
              validationErrors = _validation.validateRSAA(action);

              if (!validationErrors.length) {
                context$4$0.next = 7;
                break;
              }

              _callAPI = action[_CALL_API2['default']];

              if (_callAPI.types && Array.isArray(_callAPI.types)) {
                _requestType = _callAPI.types[0];

                if (_requestType && _requestType.type) {
                  _requestType = _requestType.type;
                }
                next({
                  type: _requestType,
                  payload: new _errors.InvalidRSAA(validationErrors),
                  error: true
                });
              }
              return context$4$0.abrupt('return');

            case 7:
              callAPI = action[_CALL_API2['default']];
              endpoint = callAPI.endpoint;
              headers = callAPI.headers;
              method = callAPI.method;
              body = callAPI.body;
              credentials = callAPI.credentials;
              bailout = callAPI.bailout;
              types = callAPI.types;
              list = callAPI.list;
              dynamicEndpoint = callAPI.dynamicEndpoint;
              _normalizeTypeDescriptors = _util.normalizeTypeDescriptors(types);
              requestType = _normalizeTypeDescriptors[0];
              successType = _normalizeTypeDescriptors[1];
              failureType = _normalizeTypeDescriptors[2];
              context$4$0.prev = 21;

              if (!(typeof bailout === 'boolean' && bailout || typeof bailout === 'function' && bailout(getState()))) {
                context$4$0.next = 24;
                break;
              }

              return context$4$0.abrupt('return');

            case 24:
              context$4$0.next = 32;
              break;

            case 26:
              context$4$0.prev = 26;
              context$4$0.t0 = context$4$0['catch'](21);
              context$4$0.next = 30;
              return _regeneratorRuntime.awrap(_util.actionWith(_extends({}, requestType, {
                payload: new _errors.RequestError('[CALL_API].bailout function failed'),
                pending: false,
                error: true
              }), [action, getState()]));

            case 30:
              context$4$0.t1 = context$4$0.sent;
              return context$4$0.abrupt('return', next(context$4$0.t1));

            case 32:
              if (!(typeof endpoint === 'function')) {
                context$4$0.next = 43;
                break;
              }

              context$4$0.prev = 33;

              endpoint = endpoint(getState());
              context$4$0.next = 43;
              break;

            case 37:
              context$4$0.prev = 37;
              context$4$0.t2 = context$4$0['catch'](33);
              context$4$0.next = 41;
              return _regeneratorRuntime.awrap(_util.actionWith(_extends({}, requestType, {
                payload: new _errors.RequestError('[CALL_API].endpoint function failed'),
                pending: false,
                error: true
              }), [action, getState()]));

            case 41:
              context$4$0.t3 = context$4$0.sent;
              return context$4$0.abrupt('return', next(context$4$0.t3));

            case 43:
              if (!(typeof headers === 'function')) {
                context$4$0.next = 54;
                break;
              }

              context$4$0.prev = 44;

              headers = headers(getState());
              context$4$0.next = 54;
              break;

            case 48:
              context$4$0.prev = 48;
              context$4$0.t4 = context$4$0['catch'](44);
              context$4$0.next = 52;
              return _regeneratorRuntime.awrap(_util.actionWith(_extends({}, requestType, {
                payload: new _errors.RequestError('[CALL_API].headers function failed'),
                pending: false,
                error: true
              }), [action, getState()]));

            case 52:
              context$4$0.t5 = context$4$0.sent;
              return context$4$0.abrupt('return', next(context$4$0.t5));

            case 54:
              context$4$0.next = 56;
              return _regeneratorRuntime.awrap(_util.actionWith(_extends({}, requestType, {
                pending: true
              }), [action, getState()]));

            case 56:
              context$4$0.t6 = context$4$0.sent;
              next(context$4$0.t6);
              context$4$0.prev = 58;

              if (!list) {
                context$4$0.next = 70;
                break;
              }

              if (!list.length) {
                context$4$0.next = 67;
                break;
              }

              callList = list.map(function (data) {
                return axios({
                  url: dynamicEndpoint(data),
                  method: method,
                  body: body,
                  data: body,
                  withCredentials: credentials,
                  headers: headers
                });
              });
              context$4$0.next = 64;
              return _regeneratorRuntime.awrap(_Promise.all(callList)['catch'](function (err) {
                throw err;
              }));

            case 64:
              res = context$4$0.sent;
              context$4$0.next = 68;
              break;

            case 67:
              res = { emptyList: true };

            case 68:
              context$4$0.next = 73;
              break;

            case 70:
              context$4$0.next = 72;
              return _regeneratorRuntime.awrap(axios({ url: endpoint, method: method, body: body, data: body, withCredentials: credentials, headers: headers }));

            case 72:
              res = context$4$0.sent;

            case 73:
              context$4$0.next = 85;
              break;

            case 75:
              context$4$0.prev = 75;
              context$4$0.t7 = context$4$0['catch'](58);

              if (!(!context$4$0.t7.response || !context$4$0.t7.response.status)) {
                context$4$0.next = 84;
                break;
              }

              context$4$0.next = 80;
              return _regeneratorRuntime.awrap(_util.actionWith(_extends({}, requestType, {
                payload: new _errors.RequestError(context$4$0.t7.message),
                error: true,
                pending: false
              }), [action, getState()]));

            case 80:
              context$4$0.t8 = context$4$0.sent;
              return context$4$0.abrupt('return', next(context$4$0.t8));

            case 84:
              res = context$4$0.t7.response;

            case 85:
              if (!(res.status >= 200 && res.status < 400 || res.length)) {
                context$4$0.next = 92;
                break;
              }

              context$4$0.next = 88;
              return _regeneratorRuntime.awrap(_util.actionWith(_extends({}, successType, {
                pending: false
              }), [action, getState(), res]));

            case 88:
              context$4$0.t9 = context$4$0.sent;
              return context$4$0.abrupt('return', next(context$4$0.t9));

            case 92:
              if (!res.emptyList) {
                context$4$0.next = 99;
                break;
              }

              context$4$0.next = 95;
              return _regeneratorRuntime.awrap(_util.actionWith(_extends({}, successType, {
                pending: false
              }), [action, getState(), res]));

            case 95:
              context$4$0.t10 = context$4$0.sent;
              return context$4$0.abrupt('return', next(context$4$0.t10));

            case 99:
              context$4$0.next = 101;
              return _regeneratorRuntime.awrap(_util.actionWith(_extends({}, failureType, {
                pending: false,
                error: true
              }), [action, getState(), res]));

            case 101:
              context$4$0.t11 = context$4$0.sent;
              return context$4$0.abrupt('return', next(context$4$0.t11));

            case 103:
            case 'end':
              return context$4$0.stop();
          }
        }, null, _this, [[21, 26], [33, 37], [44, 48], [58, 75]]);
      };
    };
  };
};

exports.apiMiddleware = apiMiddleware;

// Do not process actions without a [CALL_API] property

// Try to dispatch an error request FSA for invalid RSAAs

// Parse the validated RSAA action

// Should we bail out?

// Process [CALL_API].endpoint function

// Process [CALL_API].headers function

// We can now dispatch the request FSA

// Make the API call

// The request was malformed, or there was a network error

// Process the server response